import { __spreadArray, __assign } from 'tslib';
import { jsx, jsxs } from 'react/jsx-runtime';
import { useState, useRef } from 'react';
import { components } from 'react-select';
import { ReactComponent } from '@netapp/icons/ic_close.svg';
import { Typography } from '../Typography/index.js';
import styles from './useMultiSelect.module.scss.js';
import useRunOnceWhenTruthy from '../../hooks/useRunOnceWhenTruthy.js';
import { Popover } from '../Popover/index.js';
import { Button } from '../Button/index.js';
import { Checkbox } from '../Checkbox/index.js';

var calculateChips = function (chips, optionsWrapperWidth) {
  var _a;
  if (!Array.isArray(chips)) {
    return [chips, null];
  }
  var calculatedChips = chips === null || chips === void 0 ? void 0 : chips.reduce(function (acc, val) {
    var _a, _b, _c;
    if (((_a = val === null || val === void 0 ? void 0 : val.props) === null || _a === void 0 ? void 0 : _a.children) !== allOption.label) {
      var optionWidth = ((_c = (_b = val === null || val === void 0 ? void 0 : val.props) === null || _b === void 0 ? void 0 : _b.children) === null || _c === void 0 ? void 0 : _c.length) * 5 + 35;
      if (acc.usedWidth + optionWidth < optionsWrapperWidth) {
        acc.chips.push(val);
        acc.usedWidth += optionWidth;
      } else {
        acc.restOfOptions.push(val);
      }
    }
    return acc;
  }, {
    usedWidth: 0,
    chips: [],
    restOfOptions: []
  });
  var restOfOptionsElem = ((_a = calculatedChips.restOfOptions) === null || _a === void 0 ? void 0 : _a.length) === 0 ? null : jsx(Popover, __assign({
    trigger: 'hover',
    delayHide: 200,
    interactive: true,
    isAppendedToBody: true,
    popoverClass: styles['rest-values-tooltip-wrapper'],
    container: jsx("div", __assign({
      className: styles['rest-values']
    }, {
      children: "+".concat(calculatedChips.restOfOptions.length)
    }))
  }, {
    children: jsx("div", __assign({
      className: styles['rest-values-tooltip']
    }, {
      children: calculatedChips.restOfOptions.map(function (option) {
        var _a, _b, _c;
        return jsxs("div", __assign({
          className: styles['option']
        }, {
          children: [jsx(Typography, __assign({
            className: styles['name'],
            title: (_a = option === null || option === void 0 ? void 0 : option.props) === null || _a === void 0 ? void 0 : _a.children
          }, {
            children: (_b = option === null || option === void 0 ? void 0 : option.props) === null || _b === void 0 ? void 0 : _b.children
          })), jsx(Button, __assign({
            variant: 'icon',
            className: styles['remove'],
            onClick: option.props.removeProps.onClick
          }, {
            children: jsx(ReactComponent, {})
          }))]
        }), (_c = option === null || option === void 0 ? void 0 : option.props) === null || _c === void 0 ? void 0 : _c.children);
      })
    }))
  }));
  return [calculatedChips.chips, restOfOptionsElem];
};
var Option = function (props) {
  var _a, _b;
  var label2 = (_a = props === null || props === void 0 ? void 0 : props.data) === null || _a === void 0 ? void 0 : _a.label2;
  return jsx(components.Option, __assign({}, props, {
    children: jsxs("div", __assign({
      className: styles['multiLine-option']
    }, {
      children: [jsx(Checkbox, {
        className: styles['multiselect-checkbox'],
        isChecked: props.isSelected,
        isDisabled: props.isDisabled
      }), jsxs("div", __assign({
        className: styles['labels']
      }, {
        children: [jsx(Typography, __assign({
          className: styles['first-line'],
          color: 'unset'
        }, {
          children: props.label
        })), label2 && jsx(Typography, {
          children: (_b = props === null || props === void 0 ? void 0 : props.data) === null || _b === void 0 ? void 0 : _b.label2
        })]
      }))]
    }))
  }));
};
var allOption = {
  label: 'Select all',
  value: '*'
};
var ValueContainer = function (props) {
  var _a, _b, _c, _d;
  var normalValuesWrapper = useRef(null);
  var _e = Array.isArray(props.children) ? props.children : [null, null],
    optionsElements = _e[0],
    restElement = _e[1];
  if (!((_b = (_a = props.selectProps) === null || _a === void 0 ? void 0 : _a.multiSelectProps) === null || _b === void 0 ? void 0 : _b.isSummarizedLabel)) {
    var wrapperWidth = (((_c = normalValuesWrapper === null || normalValuesWrapper === void 0 ? void 0 : normalValuesWrapper.current) === null || _c === void 0 ? void 0 : _c.offsetWidth) || 300) - 40;
    var _f = calculateChips(optionsElements, wrapperWidth),
      normalValues = _f[0],
      rest = _f[1];
    var newChildren = jsxs("div", __assign({
      className: styles['multi-value-wrapper'],
      ref: normalValuesWrapper
    }, {
      children: [jsx("div", __assign({
        className: styles['normal-values']
      }, {
        children: normalValues
      })), rest]
    }));
    return jsxs(components.ValueContainer, __assign({}, props, {
      children: [newChildren, restElement]
    }));
  }
  var currentValues = props === null || props === void 0 ? void 0 : props.getValue();
  var optionName = ((_d = props === null || props === void 0 ? void 0 : props.selectProps) === null || _d === void 0 ? void 0 : _d.multiSelectProps).optionName;
  var text = "".concat(currentValues === null || currentValues === void 0 ? void 0 : currentValues.length, " ").concat(optionName, " selected");
  if (currentValues.length === props.options.length) {
    text = "All ".concat(optionName, " (").concat((currentValues === null || currentValues === void 0 ? void 0 : currentValues.length) - 1, ")");
  }
  return jsxs(components.ValueContainer, __assign({}, props, {
    children: [jsx(Typography, __assign({
      Component: 'span'
    }, {
      children: text
    })), restElement]
  }));
};
var MultiValue = function (props) {
  var children = props.children,
    selectProps = props.selectProps;
  var currentValues = props === null || props === void 0 ? void 0 : props.getValue();
  var _a = selectProps === null || selectProps === void 0 ? void 0 : selectProps.multiSelectProps,
    optionName = _a.optionName,
    isSummarizedLabel = _a.isSummarizedLabel;
  var text = "".concat(currentValues === null || currentValues === void 0 ? void 0 : currentValues.length, " ").concat(optionName, " selected");
  if (currentValues.some(function (val) {
    return val.value === allOption.value;
  })) {
    text = "All ".concat(optionName, " (").concat((currentValues === null || currentValues === void 0 ? void 0 : currentValues.length) - 1, ")");
  }
  var newChildren = isSummarizedLabel ? [jsx(Typography, __assign({
    Component: 'span'
  }, {
    children: text
  }), 0), Array.isArray(children) ? children[1] : null] : children;
  return jsx(components.MultiValue, __assign({}, props, {
    children: newChildren
  }));
};
var useMultiSelect = function (props, setIsMenuOpen, setTextFilter) {
  var _a;
  var _b = (props === null || props === void 0 ? void 0 : props.multiSelectProps) || {},
    _c = _b.optionName,
    optionName = _c === void 0 ? 'Options' : _c,
    _d = _b.isSelectAllAllowed,
    isSelectAllAllowed = _d === void 0 ? false : _d,
    _e = _b.isComboBox,
    isComboBox = _e === void 0 ? false : _e;
  var _f = useState([]),
    innerValue = _f[0],
    setInnerValue = _f[1];
  var options = isSelectAllAllowed ? ((_a = props.options) === null || _a === void 0 ? void 0 : _a.length) ? __spreadArray([allOption], props.options, true) : [] : props.options;
  var defaultValue = (props === null || props === void 0 ? void 0 : props.defaultValue) || [];
  useRunOnceWhenTruthy(function () {
    setInnerValue(defaultValue);
  }, !!props.options);
  var onApplyClick = function () {
    setIsMenuOpen(false);
    var newValues = innerValue.filter(function (_a) {
      var value = _a.value;
      return value !== allOption.value;
    });
    if (props.onChange) {
      props.onChange(newValues);
    }
  };
  var onCancelClick = function () {
    setIsMenuOpen(false);
    setTextFilter('');
    setInnerValue(defaultValue);
  };
  if (!(props === null || props === void 0 ? void 0 : props.multiSelectProps)) return {};
  return {
    optionName: optionName,
    options: options,
    components: {
      Option: Option,
      MultiValue: MultiValue,
      ValueContainer: ValueContainer
    },
    onApplyClick: onApplyClick,
    onCancelClick: onCancelClick,
    value: innerValue,
    onChange: function (selected, event) {
      var updateData = function (newData) {
        if (!isComboBox && (props === null || props === void 0 ? void 0 : props.onChange)) {
          props === null || props === void 0 ? void 0 : props.onChange(newData);
        }
        setInnerValue(newData);
      };
      if (selected !== null && selected.length > 0) {
        if (selected[selected.length - 1].value === allOption.value) {
          // if select all was clicked, select all options
          return updateData(options);
        }
        var result = [];
        if (selected.length === props.options.length) {
          // if all options selected ony by one, selecting also 'select All' option
          if (selected.includes(allOption)) {
            // if all was selected and one option was deselected, deselect also 'select all option
            result = selected.filter(function (option) {
              return option.value !== allOption.value;
            });
          } else if (event.action === 'select-option') {
            result = options;
          }
          return updateData(result);
        }
      }
      return updateData(selected);
    }
  };
};

export { useMultiSelect as default };
//# sourceMappingURL=useMultiSelect.js.map
