import { __makeTemplateObject, __assign, __rest } from 'tslib';
import { jsxs, jsx } from 'react/jsx-runtime';
import React, { useMemo, useRef, useState, useCallback, useEffect } from 'react';
import { ReactComponent as ReactComponent$2 } from '@netapp/icons/ic_sort.svg';
import { ReactComponent as ReactComponent$3 } from '@netapp/icons/ic_arrow_down.svg';
import { ReactComponent as ReactComponent$4 } from '@netapp/icons/ic_arrow_up.svg';
import { ReactComponent as ReactComponent$1 } from '@netapp/icons/ic_file.svg';
import { ReactComponent } from '@netapp/icons/ic_search.svg';
import _get from 'lodash/get';
import styles from './Table.module.scss.js';
import { FilterButton } from './FilterPanel.js';
import { PaginationPanel } from './PaginationPanel.js';
import { TooltipInfo } from '../TooltipInfo/index.js';
import classNames from 'classnames';
import { Typography } from '../Typography/index.js';
import { css } from '@emotion/css';
import useHover from '../../hooks/useHover.js';
import _isString from 'lodash/isString';
import { Button } from '../Button/index.js';

var renderCellValue = function (_a) {
  var column = _a.column,
    data = _a.data,
    updateRowState = _a.updateRowState,
    rowsState = _a.rowsState,
    isRowHovered = _a.isRowHovered;
  var value = _get(data, column.accessor, '');
  if (column.renderCell) {
    return column.renderCell(value, data, {
      updateRowState: updateRowState,
      rowsState: rowsState,
      isRowHovered: isRowHovered
    });
  } else {
    return jsx(Typography, __assign({
      color: "unset",
      variant: "Regular_14",
      isEllipsis: true,
      title: _isString(value) ? value : ''
    }, {
      children: value
    }));
  }
};
var Row = function (_a) {
  var _b;
  var columns = _a.columns,
    data = _a.data,
    updateRowState = _a.updateRowState,
    rowsState = _a.rowsState,
    ExpandedRow = _a.ExpandedRow,
    textFilter = _a.textFilter,
    selectionState = _a.selectionState,
    isDoubleRow = _a.isDoubleRow;
  var _c = data.cellProps || {},
    _d = _c.className,
    className = _d === void 0 ? '' : _d,
    _e = _c.isLoading,
    isLoading = _e === void 0 ? false : _e,
    isDisabled = _c.isDisabled,
    cellProps = __rest(_c, ["className", "isLoading", "isDisabled"]);
  var currentRowState = rowsState[data.id];
  var isSelected = !!(selectionState === null || selectionState === void 0 ? void 0 : selectionState.rows[data.id]);
  var _f = useHover(),
    hoverParentProps = _f.hoverParentProps,
    isRowHovered = _f.isHovered;
  return jsxs("div", __assign({
    className: classNames(styles['row'], (_b = {}, _b[styles['disabled-row']] = isDisabled, _b[styles['highlighted-row']] = (currentRowState === null || currentRowState === void 0 ? void 0 : currentRowState.isExpanded) || isSelected, _b))
  }, hoverParentProps, {
    children: [columns.map(function (column, index) {
      var _a;
      var cellValue = renderCellValue({
        column: column,
        data: data,
        updateRowState: updateRowState,
        rowsState: rowsState,
        isRowHovered: isRowHovered
      });
      var _b = column.className,
        columnClassName = _b === void 0 ? '' : _b;
      return jsx("span", __assign({
        className: classNames(styles['cell'], className, columnClassName, styles["col-".concat(index)], (_a = {}, _a[styles['last-col']] = index === columns.length - 1, _a[styles['sticky']] = column === null || column === void 0 ? void 0 : column.isSticky, _a[styles['high-row']] = isDoubleRow, _a))
      }, cellProps, {
        children: isLoading ? jsx("div", {
          className: styles['skeleton']
        }) : cellValue
      }), column.id);
    }), (currentRowState === null || currentRowState === void 0 ? void 0 : currentRowState.isExpanded) && ExpandedRow && jsx("div", __assign({
      className: styles['expanded-row-section'],
      style: {
        gridColumn: "1 / ".concat(columns.length + 1)
      }
    }, {
      children: jsx(ExpandedRow, {
        columns: columns,
        rowData: data,
        rowsState: rowsState,
        textFilter: textFilter
      })
    }))]
  }));
};
var HeaderCell = function (_a) {
  var _b;
  var column = _a.column,
    isExternalFilter = _a.isExternalFilter,
    index = _a.index,
    isLast = _a.isLast,
    columnSortState = _a.columnSortState,
    isLazyLoading = _a.isLazyLoading;
  var isSortable = column.isSortable,
    Header = column.Header,
    toggleSort = column.toggleSort,
    filterState = column.filterState,
    filterOptions = column.filterOptions,
    info = column.info,
    _c = column.infoProps,
    infoProps = _c === void 0 ? {} : _c,
    id = column.id,
    _d = column.className,
    className = _d === void 0 ? '' : _d,
    _e = column.isSticky,
    isSticky = _e === void 0 ? false : _e;
  return jsxs("div", __assign({
    className: classNames(styles['header-cell'], styles["col-".concat(index)], className, (_b = {}, _b[styles['last-col']] = isLast, _b[styles['sticky']] = isSticky, _b))
  }, {
    children: [info && jsx(TooltipInfo, __assign({
      isAppendedToBody: true,
      className: styles['table-header-tooltip']
    }, infoProps, {
      children: info
    })), jsxs(Typography, __assign({
      variant: 'Semibold_14',
      isEllipsis: true,
      className: styles['header-name']
    }, {
      children: [jsx("span", {
        children: typeof Header === 'function' ? jsx(Header, {}) : Header
      }), jsx("span", {
        children: (filterState === null || filterState === void 0 ? void 0 : filterState.activeCount) ? " (".concat(filterState.activeCount, ")") : ''
      })]
    })), (filterOptions && !isExternalFilter || isSortable) && jsxs("div", __assign({
      className: styles['filter-sort-wrapper']
    }, {
      children: [filterOptions && !isExternalFilter && jsx(FilterButton, {
        isDisabled: isLazyLoading,
        column: column
      }), isSortable && jsx(SortButton, {
        isDisabled: isLazyLoading,
        handleSort: toggleSort,
        columnSortState: columnSortState || null
      })]
    })), jsx("div", {
      className: styles['right-border']
    })]
  }), id);
};
var useHorizontalScroll = function (columns, pagesCount) {
  var _a;
  var tableBodyRef = useRef(null);
  var tableHeaderRef = useRef(null);
  var _b = useState(0),
    scrollWidth = _b[0],
    setScrollWidth = _b[1];
  var tableBodyCallbackRef = useCallback(function (node) {
    if (node !== null && columns.length) {
      tableBodyRef.current = node;
      setScrollWidth(node.scrollWidth);
    }
  }, [tableBodyRef.current, columns.length]);
  useEffect(function () {
    var _a, _b;
    if ((_a = tableBodyRef === null || tableBodyRef === void 0 ? void 0 : tableBodyRef.current) === null || _a === void 0 ? void 0 : _a.scrollWidth) {
      setScrollWidth((_b = tableBodyRef === null || tableBodyRef === void 0 ? void 0 : tableBodyRef.current) === null || _b === void 0 ? void 0 : _b.scrollWidth);
    }
  }, [(_a = tableBodyRef === null || tableBodyRef === void 0 ? void 0 : tableBodyRef.current) === null || _a === void 0 ? void 0 : _a.scrollWidth]);
  var scrollWidthClass = useMemo(function () {
    return css(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n      width: ", "px;\n    "], ["\n      width: ", "px;\n    "])), scrollWidth);
  }, [scrollWidth]);
  var scrollBottomClass = useMemo(function () {
    return css(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n      --horizontal-scroll-bottom: ", ";\n    "], ["\n      --horizontal-scroll-bottom: ", ";\n    "])), pagesCount > 1 ? '40px' : 0);
  }, [pagesCount]);
  return {
    tableBodyRef: tableBodyRef,
    tableHeaderRef: tableHeaderRef,
    scrollWidthClass: scrollWidthClass,
    scrollBottomClass: scrollBottomClass,
    tableBodyCallbackRef: tableBodyCallbackRef
  };
};
/** Data table - Should be used with useTable hook, all props go for useTable which return the gridProps (apart from className, isColumnsAutoSizeDisabled and isExternalFilter) */
var Table = React.forwardRef(function (_a, ref) {
  var _b;
  var _c;
  var tableProps = _a.tableProps,
    className = _a.className,
    isColumnsAutoSizeDisabled = _a.isColumnsAutoSizeDisabled,
    ExpandedRow = _a.ExpandedRow,
    _d = _a.isDoubleRow,
    isDoubleRow = _d === void 0 ? false : _d,
    _e = _a.isExternalFilter,
    isExternalFilter = _e === void 0 ? false : _e,
    _f = _a.lazyLoadingText,
    lazyLoadingText = _f === void 0 ? 'Loading' : _f;
  var organizedRows = tableProps.organizedRows,
    rows = tableProps.rows,
    columns = tableProps.columns,
    pagination = tableProps.pagination,
    pageSize = tableProps.pageSize,
    _g = tableProps.isHorizontalScroll,
    isHorizontalScroll = _g === void 0 ? false : _g,
    rowsState = tableProps.rowsState,
    updateRowState = tableProps.updateRowState,
    filterState = tableProps.filterState,
    selectionState = tableProps.selectionState,
    sortState = tableProps.sortState,
    isLazyLoading = tableProps.isLazyLoading;
  var gridTemplateColumns = useMemo(function () {
    return columns.reduce(function (acc, _a) {
      var width = _a.width;
      return acc + (width ? "".concat(width, " ") : isColumnsAutoSizeDisabled || isHorizontalScroll ? '1fr ' : 'auto ');
    }, '');
  }, [columns, isColumnsAutoSizeDisabled]);
  var gridTemplateColumnsClass = css(templateObject_3 || (templateObject_3 = __makeTemplateObject(["\n      grid-template-columns: ", ";\n    "], ["\n      grid-template-columns: ", ";\n    "])), gridTemplateColumns);
  var _h = useHorizontalScroll(columns, pagination === null || pagination === void 0 ? void 0 : pagination.pageCount),
    tableBodyRef = _h.tableBodyRef,
    tableHeaderRef = _h.tableHeaderRef,
    scrollWidthClass = _h.scrollWidthClass,
    scrollBottomClass = _h.scrollBottomClass,
    tableBodyCallbackRef = _h.tableBodyCallbackRef;
  return jsxs("div", __assign({
    className: classNames(styles['table-wrapper'])
  }, {
    children: [jsxs("div", __assign({
      className: classNames(styles['table'], gridTemplateColumnsClass, (_b = {}, _b[styles['fixed-cell-width']] = isHorizontalScroll, _b), className)
    }, {
      children: [jsx("div", __assign({
        className: classNames(styles['header-row'], gridTemplateColumnsClass),
        ref: tableHeaderRef
      }, {
        children: columns.map(function (column, index) {
          return jsx(HeaderCell, {
            columnSortState: (sortState === null || sortState === void 0 ? void 0 : sortState.column) === column.id ? sortState === null || sortState === void 0 ? void 0 : sortState.sortOrder : null,
            column: column,
            isExternalFilter: isExternalFilter,
            index: index,
            columns: columns,
            isLast: columns.length - 1 === index,
            isLazyLoading: isLazyLoading
          }, column.id);
        })
      })), jsx("div", __assign({
        className: classNames(styles['body'], gridTemplateColumnsClass),
        ref: tableBodyCallbackRef
      }, {
        children: (_c = pagination === null || pagination === void 0 ? void 0 : pagination.pageRows) === null || _c === void 0 ? void 0 : _c.map(function (dataRow) {
          return jsx(Row, {
            columns: columns,
            data: dataRow,
            isDoubleRow: isDoubleRow,
            ExpandedRow: ExpandedRow,
            selectionState: selectionState,
            rowsState: rowsState,
            updateRowState: updateRowState,
            textFilter: filterState === null || filterState === void 0 ? void 0 : filterState.textFilter
          }, dataRow.id);
        })
      }))]
    })), organizedRows.length === 0 && rows.length !== 0 && jsxs("div", __assign({
      className: styles['empty-table']
    }, {
      children: [jsx(ReactComponent, {}), jsx(Typography, __assign({
        variant: 'Semibold_14'
      }, {
        children: "No results"
      }))]
    })), organizedRows.length === 0 && rows.length === 0 && (isLazyLoading ? jsxs("div", __assign({
      className: classNames(styles['empty-table'], styles['smaller-table'])
    }, {
      children: [jsx(ReactComponent, {}), jsx(Typography, __assign({
        variant: 'Semibold_14'
      }, {
        children: lazyLoadingText
      }))]
    })) : jsxs("div", __assign({
      className: styles['empty-table']
    }, {
      children: [jsx(ReactComponent$1, {}), jsx(Typography, __assign({
        variant: 'Semibold_14'
      }, {
        children: "No data"
      }))]
    }))), isHorizontalScroll && jsx("div", __assign({
      className: classNames(styles['horizontal-scroll'], scrollBottomClass),
      onScroll: function (e) {
        var _a;
        var newScrollLeft = (_a = e.target) === null || _a === void 0 ? void 0 : _a.scrollLeft;
        if (tableBodyRef.current && tableHeaderRef.current) {
          tableHeaderRef.current.scrollLeft = newScrollLeft;
          tableBodyRef.current.scrollLeft = newScrollLeft;
        }
      }
    }, {
      children: jsx("div", {
        className: classNames(styles['scroll'], scrollWidthClass)
      })
    })), (pagination === null || pagination === void 0 ? void 0 : pagination.pageCount) > 1 && jsx(PaginationPanel, {
      pagination: pagination,
      pageSize: pageSize,
      totalRows: organizedRows.length
    })]
  }));
});
var SortButton = function (_a) {
  var handleSort = _a.handleSort,
    _b = _a.columnSortState,
    columnSortState = _b === void 0 ? null : _b,
    _c = _a.isDisabled,
    isDisabled = _c === void 0 ? false : _c;
  return jsxs(Button, __assign({
    variant: 'icon',
    isDisabled: isDisabled,
    className: classNames(styles['sort-button']),
    onClick: handleSort
  }, {
    children: [columnSortState === null && jsx(ReactComponent$2, {}), columnSortState === 'asc' && jsx(ReactComponent$3, {}), columnSortState === 'desc' && jsx(ReactComponent$4, {})]
  }));
};
var templateObject_1, templateObject_2, templateObject_3;

export { Table };
//# sourceMappingURL=index.js.map
