import { __assign } from 'tslib';
import { jsx } from 'react/jsx-runtime';
import React, { useContext } from 'react';
import './ThemeProvider.scss.js';
import useMediaQuery from '../../hooks/useMediaQuery.js';
import { calculateBreakpointWidth } from '../../utils/utils.js';

var DialogContext = React.createContext({
  theme: 'regular',
  screenResolution: 'largeResolution'
});
var useCalculateScreenResolution = function (isIframe) {
  var mediumResolutionBreakpoint = calculateBreakpointWidth('--Medium_Resolution_Breakpoint', isIframe);
  var smallResolutionBreakpoint = calculateBreakpointWidth('--Small_Resolution_Breakpoint', isIframe);
  var _a = useMediaQuery(["(width<= ".concat(mediumResolutionBreakpoint, ")"), "(width<= ".concat(smallResolutionBreakpoint, ")")]),
    isMediumResolution = _a[0],
    isSmallResolution = _a[1];
  if (isSmallResolution) {
    return 'smallResolution';
  }
  if (isMediumResolution) {
    return 'mediumResolution';
  }
  return 'largeResolution';
};
/** ThemeProvider should wrap your app in index.js, it is used to declare the css variables of the design system, including animations, colors and shadow */
var ThemeProvider = function (_a) {
  var children = _a.children,
    _b = _a.isIframe,
    isIframe = _b === void 0 ? true : _b;
  var screenResolution = useCalculateScreenResolution(isIframe);
  return jsx(DialogContext.Provider, __assign({
    value: {
      theme: 'regular',
      screenResolution: screenResolution
    }
  }, {
    children: children
  }));
};
var useThemeProvider = function () {
  return useContext(DialogContext);
};

export { ThemeProvider, useThemeProvider };
//# sourceMappingURL=index.js.map
