import _slicedToArray from '@babel/runtime/helpers/slicedToArray';
import _defineProperty from '@babel/runtime/helpers/defineProperty';
import { useReducer, useRef, useMemo, useCallback } from 'react';
import forEach from 'lodash/forEach';
import isEmpty from 'lodash/isEmpty';
import isArray from 'lodash/isArray';

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
var noValidator = function noValidator() {
  return {};
};
var yupValidator = function yupValidator(schema) {
  return function (state) {
    var errors = {};
    try {
      schema.validateSync(state, {
        abortEarly: false
      });
    } catch (e) {
      e.inner && forEach(e.inner, function (_ref) {
        var path = _ref.path,
          message = _ref.message;
        errors[path] = errors[path] || message;
      });
    }
    return errors;
  };
};
var init = function init(state) {
  return {
    fields: state,
    isValid: false,
    isDirty: false,
    dirty: {},
    errorState: {},
    submissionAttempted: false
  };
};
var reducer = function reducer(state, action) {
  var type = action.type,
    payload = action.payload;
  switch (type) {
    case "RESET":
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          fields: payload.fields,
          dirty: {},
          isDirty: false
        });
      }
    case "MERGE":
      {
        var fields = payload.fields,
          dirty = payload.dirty;
        return _objectSpread(_objectSpread({}, state), {}, {
          fields: _objectSpread(_objectSpread({}, state.fields), fields),
          dirty: _objectSpread(_objectSpread({}, state.dirty), dirty),
          isDirty: true
        });
      }
    case "SUBMISSION-ATTEMPTED":
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          submissionAttempted: true
        });
      }
    case "RESET-SUBMISSION-ATTEMPTED":
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          submissionAttempted: false
        });
      }
    case "VALIDITY":
      {
        return _objectSpread(_objectSpread({}, state), {}, {
          isValid: payload.isValid,
          errorState: payload.errorState
        });
      }
  }
  return state;
};
var useForm = (function (_ref2) {
  var initialState = _ref2.initialState,
    _ref2$validator = _ref2.validator,
    validate = _ref2$validator === void 0 ? noValidator : _ref2$validator;
  var _useReducer = useReducer(reducer, initialState, init),
    _useReducer2 = _slicedToArray(_useReducer, 2),
    state = _useReducer2[0],
    dispatch = _useReducer2[1];
  var stateRef = useRef({});
  var _useMemo = useMemo(function () {
      var validator = validate.validateSync ? yupValidator(validate) : validate;
      var errors = validator(state.fields);
      return {
        isValid: isEmpty(errors),
        errorState: errors
      };
    }, [validate, state.fields]),
    isValid = _useMemo.isValid,
    errorState = _useMemo.errorState;
  stateRef.current = {
    state: state,
    isValid: isValid
  };
  var resetForm = useCallback(function (state) {
    dispatch({
      type: "RESET",
      payload: {
        fields: state
      }
    });
  }, [dispatch]);
  var resetSubmissionAttempted = useCallback(function () {
    dispatch({
      type: "RESET-SUBMISSION-ATTEMPTED"
    });
  }, [dispatch]);
  var handleFormSubmit = useCallback(function (callback) {
    return function (event) {
      dispatch({
        type: "SUBMISSION-ATTEMPTED"
      });
      event.preventDefault();
      if (stateRef.current.isValid) {
        callback(event, stateRef.current.state.fields);
      }
    };
  }, [dispatch]);
  var handleFormChange = useCallback(function (_valueOrEvent, _Event) {
    if (isArray(_valueOrEvent) && !_Event) {
      var mergeState = {};
      var mergeDirty = {};
      forEach(_valueOrEvent, function (_ref3) {
        var value = _ref3.value,
          event = _ref3.event;
        mergeState[event.name] = value;
        mergeDirty[event.name] = true;
      });
      dispatch({
        type: "MERGE",
        payload: {
          fields: mergeState,
          dirty: mergeDirty
        }
      });
    } else {
      var name = null;
      var value = null;
      if (_Event) {
        name = _Event.name;
        value = _valueOrEvent;
      } else {
        var event = _valueOrEvent;
        if (event.target.type === "checkbox") {
          value = event.target.checked;
          name = event.target.value;
        } else {
          value = event.target.value;
          name = event.target.name;
        }
      }
      if (name) {
        dispatch({
          type: "MERGE",
          payload: {
            fields: _defineProperty({}, name, value),
            dirty: _defineProperty({}, name, true)
          }
        });
      } else {
        console.warn("Form event received without form name. Value ignored.");
      }
    }
  }, [dispatch]);
  var wrappedHandleFormChange = useCallback(function (name) {
    return function (value, event) {
      handleFormChange(value, _objectSpread(_objectSpread({}, event), {}, {
        name: name
      }));
    };
  }, [handleFormChange]);
  return useMemo(function () {
    return {
      resetForm: resetForm,
      handleFormSubmit: handleFormSubmit,
      handleFormChange: handleFormChange,
      wrappedHandleFormChange: wrappedHandleFormChange,
      formState: state.fields,
      dirty: state.dirty,
      isDirty: state.isDirty,
      isValid: isValid,
      formErrors: errorState,
      submissionAttempted: state.submissionAttempted,
      resetSubmissionAttempted: resetSubmissionAttempted
    };
  }, [resetForm, handleFormSubmit, handleFormChange, wrappedHandleFormChange, state, isValid, errorState, resetSubmissionAttempted]);
});

export { useForm as default, yupValidator };
//# sourceMappingURL=useForm.js.map
