import _extends from '@babel/runtime/helpers/extends';
import _slicedToArray from '@babel/runtime/helpers/slicedToArray';
import React__default, { useState, useCallback } from 'react';
import isUndefined from 'lodash/isUndefined';
import 'prop-types';

var Formidable = /*#__PURE__*/React__default.forwardRef(function (props, ref) {
  var Component = props.Component,
      form = props.form,
      name = props.name,
      _onChange = props.onChange,
      _value = props.value,
      _errorMessage = props.errorMessage,
      _showError = props.showError,
      _onBlur = props.onBlur,
      useHandleFormChangeWrapper = props.useHandleFormChangeWrapper;

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      wasFocused = _useState2[0],
      setWasFocused = _useState2[1];

  var _formHandler = formHandler({
    _onChange: _onChange,
    _value: _value,
    _errorMessage: _errorMessage,
    _showError: _showError,
    form: form,
    name: name,
    wasFocused: wasFocused,
    useHandleFormChangeWrapper: useHandleFormChangeWrapper
  }),
      onChange = _formHandler.onChange,
      value = _formHandler.value,
      errorMessage = _formHandler.errorMessage,
      showError = _formHandler.showError;

  var onBlur = useCallback(function (e) {
    setWasFocused(true);
    _onBlur && _onBlur(e);
  }, [setWasFocused, _onBlur]);
  return /*#__PURE__*/React__default.createElement(Component, _extends({
    onBlur: onBlur
  }, props, {
    onChange: onChange,
    value: value,
    error: showError && errorMessage,
    ref: ref
  }));
});

var formHandler = function formHandler(_ref) {
  var _onChange = _ref._onChange,
      _value = _ref._value,
      _showError = _ref._showError,
      _errorMessage = _ref._errorMessage,
      form = _ref.form,
      name = _ref.name,
      useHandleFormChangeWrapper = _ref.useHandleFormChangeWrapper,
      wasFocused = _ref.wasFocused;
  var onChange = _onChange;
  var value = _value;
  var errorMessage = _errorMessage;
  var showError = _showError;

  if (form && name) {
    value = _value || form.formState[name];
    onChange = _onChange || (useHandleFormChangeWrapper ? form.wrappedHandleFormChange(name) : form.handleFormChange);
    errorMessage = _errorMessage || form.formErrors[name];

    if (isUndefined(_showError)) {
      showError = (form.submissionAttempted || form.dirty[name] && wasFocused) && form.formErrors[name];
    }
  }

  return {
    onChange: onChange,
    value: value,
    errorMessage: errorMessage,
    showError: showError
  };
};

var FormField = /*#__PURE__*/React__default.forwardRef(function (props, ref) {
  return /*#__PURE__*/React__default.createElement(Formidable, _extends({}, props, {
    ref: ref
  }));
});

export { FormField as default };
//# sourceMappingURL=FormField.js.map
